
'' =================================================================================================
''
''   File....... Hexapod Leg rev 4a.spin
''   Purpose.... Create routines for leg control
''   Author..... Bob Sweeney
''   Started.... 8/26/14
''   Incorporate XBee radio input and feedback  11/27/14
''
'' =================================================================================================

con

  _clkmode = xtal1 + pll16x
  _xinfreq = 6_250_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  
con

  #1, HOME, #8, BKSP, TAB, CLREOL, CLRDN, #16, CLS      ' io formmatting control

con
  RX1           = 31
  TX1           = 30
  SDA           = 29
  SCL           = 28
  'ADC chip pins
  CS            = 27
  CLK           = 26
  DIO           = 25
  DII           = 24
  'PWM motor outputs
  femur         = 13
  tibia         = 14
  coxa          = 15
  'RGB LED color IDs
  Red           = 0
  Green         = 1
  Blue          = 2
  Yellow        = 3
  White         = 4
  Orange        = 5
  LtBlue        = 6
  Off           = 7
  
  Degrees       = (180.0/pi)
  Radians       = (pi / 180.0)
  'PWM to stop motor
  stopmotor     = 1500
  'interval in msec for ramp timing
  interval      = 40
  'unique number for leg
  leg           = 6                                                                    'default values
  CR            = 13
  LF            = 10


obj
  adc           : "ADC_Input_Driver"
  dc            : "jm_servo8"
  F             : "Float32"
  io            : "pcFullDuplexSerial4FC" '1 COG for 4 serial ports
  re            : "regex"

var
  long i, j, cptr, ptr, arg, Null[1], femurBufa[10], tibiaBufa[10], coxaBufa[10]
  long array[11]
  long CoxaStack[250], TibiaStack[250], FemurStack[250], adcstack[50]
  word Femurcounter, TibiaCounter, CoxaCounter
  word CoxaAngle, CoxaADC, CoxaDone, CoxaDisable, CAdcInt
  word TibiaAngle, TibiaADC, TibiaDone, TibiaDisable, TAdcInt
  word FemurAngle, FemurADC, FemurDone, FemurDisable, FAdcInt
  word FemurTest, TibiaTest, CoxaTest
  byte buffer[10], femurcolor, tibiacolor, coxacolor, legcolor
  byte Rx, cksum, femurBufb[20], tibiaBufb[20], coxaBufb[20]

pub Start | n
  F.Start
  io.Init
  'uarts.AddPort(vm_port,vm_rx,vm_tx,vm_cts,vm_rts,UARTS#DEFAULTTHRESHOLD,UARTS#NOMODE,UARTS#BAUD9600)
  io.AddPort(0,31,30,io#PINNOTUSED,io#PINNOTUSED,io#DEFAULTTHRESHOLD,io#NOMODE,io#BAUD115200) 'Serial terminal output
  io.AddPort(1,1,0,  io#PINNOTUSED,io#PINNOTUSED,io#DEFAULTTHRESHOLD,io#NOMODE,io#BAUD115200)                                                      'IO port to master
  io.Start                                                                      'Start the ports
  dc.start(3, 13)                                                               'start motor PWM
  cognew(GetADC, @adcstack)                                                     'run ADC on its own cog

  dira[femur] := 1                                                             'femur motor output
  dira[tibia] := 1                                                             'tibia motor output
  dira[coxa]  := 1                                                             'coxa motor output
  'set test value = 1 for display to serial terminal 
  FemurTest := 1
  TibiaTest := 0
  CoxaTest  := 0
  
  pauseMSec(500)                                                              'let objects finish loading

  FemurDone := 0                                                               'leg movement done if = 0
  TibiaDone := 0
  CoxaDone  := 0
  'disable angle inputs until initialization complete
  FemurDisable := 1
  TibiaDisable := 1
  CoxaDisable  := 1
  io.tx(0,16)

  'get ADC interval per degree of movement
  'calculate ADC value using slope of line, y=mx+b where m = y1-y0/x1-x0
  FAdcInt := f.fTrunc(f.fdiv(f.fsub(f.ffloat(FemurMaxADC),f.ffloat(FemurMinADC)), f.fsub(f.ffloat(FemurMaxAngle), f.ffloat(FemurMinAngle))))
  TAdcInt := f.fTrunc(f.fdiv(f.fsub(f.ffloat(TibiaMaxADC),f.ffloat(TibiaMinADC)), f.fsub(f.ffloat(TibiaMaxAngle), f.ffloat(TibiaMinAngle))))
  CAdcInt := f.fTrunc(f.fdiv(f.fsub(f.ffloat(CoxaMaxADC),f.ffloat(CoxaMinADC)), f.fsub(f.ffloat(CoxaMaxAngle), f.ffloat(CoxaMinAngle))))
  F.Stop                                                                        'FP not used again
  
  cognew(FemurMotorRun, @femurStack)
  cognew(TibiaMotorRun, @tibiaStack)
  cognew(CoxaMotorRun, @coxaStack)

  'enable motors after pause to ensure clean cog startup
  pauseMSec(500)                                                                'let objects load
  
  FemurAngle := 90
  TibiaAngle := 90
  CoxaAngle  := 90
  
  FemurDisable := 0
  TibiaDisable := 0
  CoxaDisable  := 0
  
  EnterCommand

pub EnterCommand | teststr, pattern, resaddr, rslt, n, legnumber, command, parameter  
' input data from master
' input string format: "$,1,2,345"
' routine derived from Perry James Mole GPS parser for GPS decoding - see thread on GPS Parsing Problems 9/9/2013
  io.tx(0,16)
  Null[0] := 0
  repeat
    longfill(buffer,10,0)
    repeat while Rx <>= "$"
      Rx := io.rx(1)
    cptr := 0
    repeat while Rx <>= CR
      Rx := io.rx(1)
      if RX == ","
        buffer[cptr++] := 0
      else
        buffer[cptr++] := Rx

    if buffer[1] == "6"
      if buffer[3] == "1"
        io.str(0,string(13,"Femur: "))
        parameter := @buffer[5]
        io.str(0,parameter)
        if FemurDone == 0
          FemurAngle := parameter
          io.str(0,string("  FemurAngle: "))
          io.dec(0,FemurAngle)
          FemurDone := 1
      if buffer[3] == "2"
'        io.str(0,string(13,"Tibia: "))
        parameter := @buffer[5]
'        io.str(0,parameter)
      if buffer[3] == "3"
'        io.str(0,string(13,"Coxa: "))
        parameter := @buffer[5]
'        io.str(0,parameter)
                                              
    ' report all motors complete movement
    if FemurDone == 0 AND TibiaDone == 0 AND CoxaDone == 0
      io.str(0,string(13,"All Motors Done.",13))

    'send back color values
'    io.str(0,string(13,"FemurColor: "))
'    io.dec(0,femurcolor)
'    io.tx(1,femurcolor)

    'build output string containing leg, action and LED color value
    'send out for each motor
{    repeat i from 1 to 3
      io.str(1,string("$,"))
      io.dec(1,legnumber)
      io.str(1,string(","))
      io.dec(1,i)
      io.str(1,string(","))
      case i
        1: io.dec(1,femurcolor)
        2: io.dec(1,tibiacolor)
        3: io.dec(1,tibiacolor)
      io.tx(1,13)
}

pub DecodeInput(cmd, prmtr)
'decode value in leg, motor, parameter to specific leg movement
  case cmd
    1: if FemurDone == 0                                'no motor updates until motor done moving
         FemurAngle := prmtr
'         io.str(0,string("  FemurAngle: "))
'         io.dec(0,FemurAngle)
         FemurDone := 1
    2: if TibiaDone == 0 
         TibiaAngle := prmtr
'         io.str(0,string("  TibiaAngle: "))
'         io.dec(0,TibiaAngle)
         TibiaDone := 1
    3: if CoxaDone == 0
         CoxaAngle := prmtr    
'         io.str(0,string("  CoxaAngle: "))
'         io.dec(0,CoxaAngle)
         CoxaDone := 1

PUB getstr(port,stringptr) | index
  '' copied from FullDuplexSerial
  '' Gets zero terminated string and stores it, starting at the stringptr memory address
  index~
  repeat until ((byte[stringptr][index++] := io.rx(port)) == 13)
  byte[--index]~

pub FemurmotorRun | Angle, TargetADC, oldangle, minsize, resetsize, moveangle
 ' input angle value and the motor moves to that angle
  dira[femur] := 1                                                              'femur motor output
  repeat
    repeat while FemurDisable == 0 'AND FemurDone == 1
      pauseMSec(250)
      
      Angle := FemurAngle
      if oldAngle <> Angle                                                 'skip if angle does not change
'        io.str(0,String(13, "FemurAngle "))
'        io.dec(0,angle)                                    
        if Angle == 30                                                    'special - close up leg
          minsize := 1
        if Angle => FemurMinAngle AND minsize == 1                        'special - reset leg from ADC = 0
          resetsize := 1
          minsize := 0  
        Angle := FemurMinAngle #> Angle <# FemurMaxAngle              'constrain input 
        oldAngle := Angle
        'determine # degrees to move to new position
        moveangle := Angle - FemurMinAngle
        'no movement unless 1 or more degrees
        if moveangle > 0 OR minsize == 1 OR resetsize == 1
          'calculate ADC value for input angle
          io.str(0,String(13, "Femur Test 1: "))
          io.dec(0,moveangle)                                    
          TargetADC := FemurMinADC + (FAdcInt * moveangle)                      'calc ADC value to move to
          if FemurTest == 1
            io.str(0,String(13, "TargetADC "))
            io.dec(0,TargetADC)                                    
            io.str(0,String("  - FemurAngle1: "))
            io.dec(0,Angle)                                    
            'special reset when recovery from min leg size and ADC = 0
          if resetsize == 1                                                     '
            dc.set(0, 1000, 0)
            pauseMSec(500)
            dc.set(0, stopmotor, 0)
            resetsize := 0
          FemurRamp(TargetADC, Angle)
          if minsize == 1                                                       'special leg storage position
            dc.set(0, 2000, 0)
            pauseMSec(500)
            dc.set(0, stopmotor, 0)
            minsize := 0
            if FemurTest == 1
              io.str(0,String(13, "Femur Done 1"))
        else
          if FemurTest == 1
            io.str(0,String(13, "Femur Done 2"))
        io.tx(1,0)
      FemurDone := 0
           

pub FemurRamp(TargetADC, Angle) | StartADC, ADChalf, adc1, actualADC, oldADC, rampdirection, distance, speed, t2, t1 
  'initialize and determine motor direction
  femurDone := 1                                                                'flag that leg is moving
  ActualADC := FemurADC                                                         'get current ADC value
  distance := ||(ActualADC - TargetADC)                                         'absolute ADC distance to move
  if distance > 25                                                              'check if in deadzone
    if (ActualADC - TargetADC) > 0                                              'determine motor direction
      femurcolor := Blue
      rampdirection := 2
    else
      femurcolor := Yellow
      rampdirection := 1
  else
    dc.set(0, stopmotor, 0)                                                     'stop motor, inside deadzone
'    femurcolor := Red
    if FemurTest == 1
      io.str(0,String(13, "Femur Done 4"))
    return
'up ramp 
  Femurcounter := 1                                                             'check for stopped motor
  repeat t2 from 8 to 0                                                         '9 steps for s-curve
    speed := PWMcalcFemur(t2, rampdirection)                                         'get speed for s-curve value 
    dc.set(0, speed, 0)                                                         'start motor at speed
    pauseMSec(interval)                                                         'setup scurve timer interval                                                         
    ActualADC := FemurADC                                                       'get current ADC value
    if FemurCounter == 1                                                        'check if ADC not changing
      oldADC := ActualADC                                                       'reset check ADC value
    if FemurStopError(oldADC, ActualADC)                                        'error if ADC not changing
      dc.set(0, stopmotor, 0)
      femurcolor := Red
      if FemurTest == 1
        io.str(0,String(13, "Femur Done 5"))
        io.str(0,String(13, "Error: up ramp "))
        io.dec(0,ActualADC)                                    
      return                                                                    'exit ramp procedure                        
    if t2 == 8                                                                  'get initial position data after motor start
      ActualADC := FemurADC                                                     'get current ADC value
      startADC := ActualADC                                                     'save starting ADC value
      distance := ||(ActualADC - TargetADC)                                     'absolute ADC distance to move
      case rampdirection
        1: ADChalf := ActualADC + (distance/3)                                  'get ramp stop mark
        2: ADChalf := ActualADC - (distance/3)                                  'get ramp stop mark
      if FemurTest == 1
        io.str(0,String(13, "Start: "))
        io.str(0,String("Angle: "))
        io.dec(0,Angle)                                    
        io.str(0,String("  -ActualADC: "))
        io.dec(0,ActualADC)                                    
        io.str(0,String( "  -TargetADC: "))
        io.dec(0,TargetADC)                                    
        io.str(0,String( "  -StartADC: "))
        io.dec(0,StartADC)                                    
        io.str(0,String( "  -adcHalf: "))
        io.dec(0,adcHalf)                                    

    case rampdirection
      1: if ActualADC > ADChalf                                                 'if at halfway point, stop ramp
           if FemurTest == 1
             io.str(0,String(13, "Stop ramp: "))
             io.dec(0,Angle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -T2: "))
             io.dec(0,T2)                                    
             io.str(0,String( "  -Dir: "))
             io.dec(0,rampdirection)                                    
             io.str(0,String( "  -ADChalf: "))
             io.dec(0,ADChalf)                                    
           quit
      2: if ActualADC < ADChalf                                                 'if at halfway point, stop ramp
           if FemurTest == 1
             io.str(0,String(13, "Stop ramp: "))
             io.dec(0,Angle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -T2: "))
             io.dec(0,T2)                                    
             io.str(0,String( "  -Dir: "))
             io.dec(0,rampdirection)                                    
             io.str(0,String( "  -ADChalf: "))
             io.dec(0,ADChalf)                                    
           quit
    if FemurTest == 1
      io.str(0,String(13, "Angle: "))
      io.dec(0,Angle)                                    
      io.str(0,String("  -ActualADC: "))
      io.dec(0,ActualADC)                                    
      io.str(0,String( "  -T2: "))
      io.dec(0,T2)                                    
      io.str(0,String( "  -Dir: "))
      io.dec(0,rampdirection)                                    
      io.str(0,String( "  -TargetADC: "))
      io.dec(0,TargetADC)                                    
   
  'run at full speed until same ADC interval (+fudge) used to start is reached
  oldADC := ActualADC
  case rampdirection
    1: adc1 := TargetADC - 250                                                  'average value for full stop
       repeat while ActualADC < adc1                                            'maintain full speed
         ActualADC := FemurADC                                                  'get current ADC value
         pauseMSec(interval)                                                    'setup scurve timer interval
         if FemurCounter == 1
           oldADC := ActualADC
         if FemurStopError(oldADC, ActualADC)                                    'error if ADC not changing
           dc.set(0, stopmotor, 0)
           femurcolor := Red
           if FemurTest == 1
             io.str(0,String(13, "Femur Done 6"))
             io.str(0,String(13, "Error: Full speed, Dir 1 "))
             io.dec(0,ActualADC)                                    
           return
         if FemurTest == 1  
           io.str(0,String(13, "Angle: "))
           io.dec(0,Angle)                                    
           io.str(0,String("  -ActualADC: "))
           io.dec(0,ActualADC)                                    
           io.str(0,String( "  -adc1: "))
           io.dec(0,adc1)                                    
           io.str(0,String( "  -Dir: "))
           io.dec(0,rampdirection)                                    
           io.str(0,String( "  -TargetADC: "))
           io.dec(0,TargetADC)                                    
           io.str(0,String( "  -Speed: "))
           io.dec(0,speed)                                    
    2: adc1 := TargetADC + 250                                                  'average value for full stop
       repeat while ActualADC > adc1                                            'maintain full speed
         ActualADC := FemurADC                                                  'get current ADC value
         pauseMSec(interval)                                                    'setup scurve timer interval
         if FemurCounter == 1
           oldADC := ActualADC
         if FemurStopError(oldADC, ActualADC)                                    'error if ADC not changing
           dc.set(0, stopmotor, 0)
           femurcolor := Red
           if FemurTest == 1
             io.str(0,String(13, "Femur Done 7"))
             io.str(0,String(13, "Error: Full speed, Dir 2 "))
             io.dec(0,ActualADC)                                    
           return
         if FemurTest == 1  
           io.str(0,String(13, "Angle: "))
           io.dec(0,Angle)                                    
           io.str(0,String("  -ActualADC: "))
           io.dec(0,ActualADC)                                    
           io.str(0,String( "  -adc1: "))
           io.dec(0,adc1)                                    
           io.str(0,String( "  -Dir: "))
           io.dec(0,rampdirection)                                    
           io.str(0,String( "  -TargetADC: "))
           io.dec(0,TargetADC)                                    
           io.str(0,String( "  -Speed: "))
           io.dec(0,speed)                                    
   
  'down ramp
  t2 := t2 + 2                                                                  'reset counter to 0
  t2 := 0 #> t2 <# 8                                                            'constrain input 
  oldADC := ActualADC
  repeat t1 from t2 to 8                                                        'start where ramp stopped
    speed := PWMcalcFemur(t1, rampdirection)                                         'get speed for s-curve value 
    dc.set(0, speed, 0)
    pauseMSec(interval)                                                         'setup scurve timer interval
    ActualADC := FemurADC                                                       'get current ADC value
    if FemurCounter == 1
      oldADC := ActualADC
    if FemurStopError(oldADC, ActualADC)                                    'error if ADC not changing
      dc.set(0, stopmotor, 0)
      femurcolor := Red
      if FemurTest == 1
        io.str(0,String(13, "Femur Done 8"))
        io.str(0,String(13, "Error: Down ramp "))
        io.dec(0,ActualADC)                                    
      return
     
    case rampdirection
      1: if ActualADC => TargetADC                                              'at target value, stop 'motor
           speed := 1500
           dc.set(0, speed, 0)
'           femurcolor := Green
           if FemurTest == 1
             io.str(0,String(13, "Femur Done 9"))
             io.str(0,String(13, "Stop: "))
             io.dec(0,Angle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -TargetADC: "))
             io.dec(0,TargetADC)                                    
             io.str(0,String( "  -Speed: "))
             io.dec(0,speed)                                    
             io.str(0,String( "  -T1: "))
             io.dec(0,t1)
             io.str(0,String( "  -Dir: "))
             io.dec(0,t2)
           quit 
      2: if ActualADC =< TargetADC                                              'at target value, stop motor
           speed := 1500
           dc.set(0, speed, 0)
'           femurcolor := Green
           if FemurTest == 1
             io.str(0,String(13, "Femur Done 10"))
             io.str(0,String(13, "Stop: "))
             io.dec(0,Angle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -TargetADC: "))
             io.dec(0,TargetADC)                                    
             io.str(0,String( "  -Speed: "))
             io.dec(0,speed)                                    
             io.str(0,String( "  -T1: "))
             io.dec(0,t1)
             io.str(0,String( "  -Dir: "))
             io.dec(0,t2)
           quit 
    if FemurTest == 1
      io.str(0,String(13, "Angle: "))
      io.dec(0,Angle)                                    
      io.str(0,String("  -ActualADC: "))
      io.dec(0,ActualADC)                                    
      io.str(0,String( "  -TargetADC: "))
      io.dec(0,TargetADC)                                    
      io.str(0,String( "  -Speed: "))
      io.dec(0,speed)                                    
      io.str(0,String( "  -T1: "))
      io.dec(0,t1)
      io.str(0,String( "  -Dir: "))
      io.dec(0,t2)
  'ensure stop 'motor at end of rampdown
  dc.set(0, stopmotor, 0)
  femurcolor := Off
  if FemurTest == 1
    io.str(0,String(13, "Finish: "))
    io.str(0,String("Angle: "))
    io.dec(0,Angle)                                    
    io.str(0,String("  -ActualADC: "))
    io.dec(0,ActualADC)                                    
    io.str(0,String( "  -TargetADC: "))
    io.dec(0,TargetADC)                                    
    io.str(0,String(13))
  if FemurTest == 1
    io.str(0,String(13, "Femur Done 11"))
   

pub FemurStopError(iADC, aADC)
  FemurCounter := FemurCounter + 1                                                                    'count to 5
  if FemurCounter == 1                                                                     'at start get ADC value
    iADC := aADC
  if FemurCounter == 5                                                                     'max count 5
    FemurCounter := 0
    if iADC == aADC                                                             'error, ADC not changing
      Result := true
   

pub TibiamotorRun | TargetADC, moveangle, oldangle
 ' input ADC value via io and the motor moves to that angle
  dira[tibia] := 1                                                             'tibia motor output
  repeat
    repeat while TibiaDisable == 0 AND TibiaDone == 1
'      pauseMSec(250)
      if oldAngle <> TibiaAngle                                                 'skip if angle does not change
        TibiaAngle := TibiaMinAngle #> TibiaAngle <# TibiaMaxAngle                  'constrain input 
        oldAngle := TibiaAngle
        'determine # degrees to move to new position
        moveangle := TibiaAngle - TibiaMinAngle
        'no movement unless 1 or more degrees
        if moveangle => 0 
          'calculate ADC value for input angle
          TargetADC := TibiaMinADC + (TAdcInt * moveangle)
          if TibiaTest == 1
            io.str(0,String(13, "TibiaAngle: "))
            io.dec(0,TibiaAngle)                                    
            io.str(0,String("  - moveangle: "))
            io.dec(0,moveangle)                                    
            io.str(0,String("  - TargetADC: "))
            io.dec(0,TargetADC)                                    
            io.str(0,String("  - TAdcInt: "))
            io.dec(0,TAdcInt)                                    
          TibiaRamp(TargetADC)
          if TibiaTest == 1
            io.str(0,String(13, "Tibia Done 1"))
        if TibiaTest == 1
          io.str(0,String(13, "Tibia Done 2"))
      TibiaDone := 0
         

pub TibiaRamp(TargetADC) | StartADC, ADChalf, adc1, actualADC, oldADC, rampdirection, distance, speed, t2, t1 
  'initialize and determine motor direction
  TibiaDone := 1                                                                'flag that leg is moving
  ActualADC := TibiaADC                                                      'get current ADC value
  distance := ||(ActualADC - TargetADC)                                       'absolute ADC distance to move
  if distance > 10                                                              'check if in deadzone
    if (ActualADC - TargetADC) > 0                                            'determine motor direction
      tibiaColor := LtBlue
      rampdirection := 2
    else
      tibiaColor := Orange
      rampdirection := 1
  else
    dc.set(1, stopmotor, 0)                                                     'stop motor, inside deadzone
    tibiaColor := Red
    if TibiaTest == 1
      io.str(0,String(13, "Tibia Done 3"))
    return
'up ramp 
  Tibiacounter := 1                                                             'check for stopped motor
  repeat t2 from 8 to 0                                                         '9 steps for s-curve
    speed := PWMcalcTibia(t2, rampdirection)                                         'get speed for s-curve value 
    dc.set(1, speed, 0)                                                         'start motor at speed
    pauseMSec(interval)                                                         'setup scurve timer interval                                                         
    ActualADC := TibiaADC                                                       'get current ADC value
    if TibiaCounter == 1                                                        'check if ADC not changing
      oldADC := ActualADC                                                       'reset check ADC value
    if TibiaStopError(oldADC, ActualADC)                                        'error if ADC not changing
      dc.set(1, stopmotor, 0)
      tibiaColor := Red
      TibiaCounter := 0
      if TibiaTest == 1
        io.str(0,String(13, "Tibia Done 4"))
      if TibiaTest == 1
        io.str(0,String(13, "Error: up ramp "))
        io.dec(0,ActualADC)                                    
      return                                                                    'exit ramp procedure                        
    if t2 == 8                                                                  'get initial position data after motor start
      ActualADC := TibiaADC                                                     'get current ADC value
      startADC := ActualADC                                                     'save starting ADC value
      distance := ||(ActualADC - TargetADC)                                     'absolute ADC distance to move
      case rampdirection
        1: ADChalf := ActualADC + (distance/4)                                  'get halfway mark
        2: ADChalf := ActualADC - (distance/4)                                  'get halfway mark
      if TibiaTest == 1
        io.str(0,String(13, "Start: "))
        io.str(0,String("Tibia Angle: "))
        io.dec(0,TibiaAngle)                                    
        io.str(0,String("  -ActualADC: "))
        io.dec(0,ActualADC)                                    
        io.str(0,String( "  -TargetADC: "))
        io.dec(0,TargetADC)                                    
        io.str(0,String( "  -StartADC: "))
        io.dec(0,StartADC)                                    
        io.str(0,String( "  -adcHalf: "))
        io.dec(0,adcHalf)                                    

    case rampdirection
      1: if ActualADC > ADChalf                                             'if at halfway point, stop ramp
           if TibiaTest == 1
             io.str(0,String(13, "Stop Tibia ramp: "))
             io.dec(0,TibiaAngle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -T2: "))
             io.dec(0,T2)                                    
             io.str(0,String( "  -Dir: "))
             io.dec(0,rampdirection)                                    
             io.str(0,String( "  -ADChalf: "))
             io.dec(0,ADChalf)                                    
           quit
      2: if ActualADC < ADChalf                                             'if at halfway point, stop ramp
           if TibiaTest == 1
             io.str(0,String(13, "Stop Tibia ramp: "))
             io.dec(0,TibiaAngle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -T2: "))
             io.dec(0,T2)                                    
             io.str(0,String( "  -Dir: "))
             io.dec(0,rampdirection)                                    
             io.str(0,String( "  -ADChalf: "))
             io.dec(0,ADChalf)                                    
           quit
    if TibiaTest == 1
      io.str(0,String(13, "Tibia Angle: "))
      io.dec(0,TibiaAngle)                                    
      io.str(0,String("  -ActualADC: "))
      io.dec(0,ActualADC)                                    
      io.str(0,String( "  -T2: "))
      io.dec(0,T2)                                    
      io.str(0,String( "  -Dir: "))
      io.dec(0,rampdirection)                                    
      io.str(0,String( "  -TargetADC: "))
      io.dec(0,TargetADC)                                    
       
  'run at full speed until same ADC interval (+fudge) used to start is reached
  oldADC := ActualADC
  case rampdirection
    1: adc1 := TargetADC - 250                                                  'average value for full stop
       repeat while ActualADC < adc1                                            'maintain full speed
         ActualADC := TibiaADC                                                  'get current ADC value
         pauseMSec(interval)                                                    'setup scurve timer interval
         if TibiaCounter == 1
           oldADC := ActualADC
         if TibiaStopError(oldADC, ActualADC)                                    'error if ADC not changing
           dc.set(1, stopmotor, 0)
           tibiaColor := Red
           if TibiaTest == 1
             io.str(0,String(13, "Tibia Done 5"))
           if TibiaTest == 1
             io.str(0,String(13, "Error: Full speed, Dir 1 "))
             io.dec(0,ActualADC)                                    
           return
         if TibiaTest == 1  
           io.str(0,String(13, "Tibia Angle: "))
           io.dec(0,TibiaAngle)                                    
           io.str(0,String("  -ActualADC: "))
           io.dec(0,ActualADC)                                    
           io.str(0,String( "  -adc1: "))
           io.dec(0,adc1)                                    
           io.str(0,String( "  -Dir: "))
           io.dec(0,rampdirection)                                    
           io.str(0,String( "  -TargetADC: "))
           io.dec(0,TargetADC)                                    
           io.str(0,String( "  -Speed: "))
           io.dec(0,speed)                                    
    2: adc1 := TargetADC + 250                                                  'average value for full stop
       repeat while ActualADC > adc1                                            'maintain full speed
         ActualADC := TibiaADC                                                  'get current ADC value
         pauseMSec(interval)                                                    'setup scurve timer interval
         if TibiaCounter == 1
           oldADC := ActualADC
         if TibiaStopError(oldADC, ActualADC)                                    'error if ADC not changing
           dc.set(1, stopmotor, 0)
           tibiaColor := Red
           TibiaCounter := 0
           if TibiaTest == 1
             io.str(0,String(13, "Tibia Done 6"))
           if TibiaTest == 1
             io.str(0,String(13, "Error: Full speed, Dir 2 "))
             io.dec(0,ActualADC)                                    
           return
         if TibiaTest == 1  
           io.str(0,String(13, "Tibia Angle: "))
           io.dec(0,TibiaAngle)                                    
           io.str(0,String("  -ActualADC: "))
           io.dec(0,ActualADC)                                    
           io.str(0,String( "  -adc1: "))
           io.dec(0,adc1)                                    
           io.str(0,String( "  -Dir: "))
           io.dec(0,rampdirection)                                    
           io.str(0,String( "  -TargetADC: "))
           io.dec(0,TargetADC)                                    
           io.str(0,String( "  -Speed: "))
           io.dec(0,speed)                                    
   
  'down ramp
  t2 := t2 + 2                                                                  'reset counter to 0
  t2 := 0 #> t2 <# 8                                                            'constrain input 
  oldADC := ActualADC
  repeat t1 from t2 to 8                                                        'start where ramp stopped
    speed := PWMcalcTibia(t1, rampdirection)                                         'get speed for s-curve value 
    dc.set(1, speed, 0)
    pauseMSec(interval)                                                         'setup scurve timer interval
    ActualADC := TibiaADC                                                       'get current ADC value
    if TibiaCounter == 1
      oldADC := ActualADC
    if TibiaStopError(oldADC, ActualADC)                                    'error if ADC not changing
      dc.set(1, stopmotor, 0)
      tibiaColor := Red
      TibiaCounter := 0
      if TibiaTest == 1
        io.str(0,String(13, "Tibia Done 7"))
      if TibiaTest == 1
        io.str(0,String(13, "Error: Down ramp "))
        io.dec(0,ActualADC)                                    
      return
     
    case rampdirection
      1: if ActualADC => TargetADC                                              'at target value, stop 'motor
           speed := 1500
           dc.set(1, speed, 0)
           tibiaColor := Green
           result := 0
           if TibiaTest == 1
             io.str(0,String(13, "Tibia Done 8"))
           if TibiaTest == 1
             io.str(0,String(13, "Tibia Stop: "))
             io.dec(0,TibiaAngle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -TargetADC: "))
             io.dec(0,TargetADC)                                    
             io.str(0,String( "  -Speed: "))
             io.dec(0,speed)                                    
             io.str(0,String( "  -T1: "))
             io.dec(0,t1)
             io.str(0,String( "  -Dir: "))
             io.dec(0,t2)
           quit 
      2: if ActualADC =< TargetADC                                              'at target value, stop motor
           speed := 1500
           dc.set(1, speed, 0)
           tibiaColor := Green
           if TibiaTest == 1
             io.str(0,String(13, "Tibia Done 9"))
           if TibiaTest == 1
             io.str(0,String(13, "Tibia Stop: "))
             io.dec(0,TibiaAngle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -TargetADC: "))
             io.dec(0,TargetADC)                                    
             io.str(0,String( "  -Speed: "))
             io.dec(0,speed)                                    
             io.str(0,String( "  -T1: "))
             io.dec(0,t1)
             io.str(0,String( "  -Dir: "))
             io.dec(0,t2)
           quit 
    if TibiaTest == 1
      io.str(0,String(13, "Tibia Angle: "))
      io.dec(0,TibiaAngle)                                    
      io.str(0,String("  -ActualADC: "))
      io.dec(0,ActualADC)                                    
      io.str(0,String( "  -TargetADC: "))
      io.dec(0,TargetADC)                                    
      io.str(0,String( "  -Speed: "))
      io.dec(0,speed)                                    
      io.str(0,String( "  -T1: "))
      io.dec(0,t1)
      io.str(0,String( "  -Dir: "))
      io.dec(0,t2)
    'ensure stop 'motor at end of rampdown
  dc.set(1, stopmotor, 0)
  tibiaColor := Green
  if TibiaTest == 1
    io.str(0,String(13, "Finish: "))
    io.str(0,String("Tibia Angle: "))
    io.dec(0,TibiaAngle)                                    
    io.str(0,String("  -ActualADC: "))
    io.dec(0,ActualADC)                                    
    io.str(0,String( "  -TargetADC: "))
    io.dec(0,TargetADC)                                    
    io.str(0,String(13))
  if TibiaTest == 1
    io.str(0,String(13, "Tibia Done 10"))
   

pub TibiaStopError(iADC, aADC)
  TibiaCounter := TibiaCounter + 1                                                                    'count to 5
  if TibiaCounter == 1                                                                     'at start get ADC value
    iADC := aADC
  if TibiaCounter == 5                                                                     'max count 5
    TibiaCounter := 0
    if iADC == aADC                                                             'error, ADC not changing
      Result := true
   

pub CoxaMotorRun | TargetADC, moveangle, oldangle
 ' input ADC value via io and the motor moves to that angle
  dira[coxa]  := 1                                                              'coxa motor output
  repeat
    repeat while CoxaDisable == 0 AND CoxaDone == 1
'      pauseMSec(250)
      if oldAngle <> CoxaAngle                                                 'skip if angle does not change
        CoxaAngle := CoxaMinAngle #> CoxaAngle <# CoxaMaxAngle                  'constrain input 
        oldAngle := CoxaAngle
        'determine # degrees to move to new position
        moveangle := CoxaAngle - CoxaMinAngle
        'no movement unless 1 or more degrees
        if moveangle > 0 
          'calculate ADC value for input angle
          TargetADC := CoxaMinADC + (CAdcInt * moveangle)
        if CoxaTest == 1
          io.str(0,String(13, "TargetADC "))
          io.dec(0,TargetADC)                                    
          io.str(0,String("  - moveAngle: "))
          io.dec(0,moveangle)                                    
        if TargetADC > CoxaMaxADC OR TargetADC < CoxaMinADC                     'ADC value out of range, error
          dc.set(2, stopmotor, 0)
          coxaColor := Red
          CoxaDone := 0
          if CoxaTest == 1
            io.str(0,String(13, "Coxa Done 1"))
            io.str(0,String(13, "Error: TargetADC "))
            io.dec(0,TargetADC)                                    
            io.str(0,String("  - CoxaAngle: "))
            io.dec(0,CoxaAngle)                                    
        else
          CoxaRamp(TargetADC)
        if CoxaTest == 1
          io.str(0,String(13, "Coxa Done 2"))
      CoxaDone := 0


pub CoxaRamp(TargetADC) | StartADC, ADChalf, adc1, actualADC, oldADC, rampdirection, distance, speed, t2, t1 
  'initialize and determine motor direction
  CoxaDone := 1                                                                'flag that leg is moving
  ActualADC := CoxaADC                                                      'get current ADC value
  distance := ||(ActualADC - TargetADC)                                       'absolute ADC distance to move
  if distance > 10                                                              'check if in deadzone
    if (ActualADC - TargetADC) > 0                                            'determine motor direction
      coxaColor := LtBlue
      rampdirection := 1
    else
      coxaColor := Orange
      rampdirection := 2
  else
    dc.set(2, stopmotor, 0)                                                     'stop motor, inside deadzone
    coxaColor := Blue
    if CoxaTest == 1
      io.str(0,String(13, "Coxa Done 3"))
      io.str(0,String(13, "Inside deadzone: "))
      io.dec(0,ActualADC)                                    
    return
'up ramp 
  Coxacounter := 1                                                             'check for stopped motor
  repeat t2 from 8 to 0                                                         '9 steps for s-curve
    speed := PWMcalcCoxa(t2, rampdirection)                                         'get speed for s-curve value 
    dc.set(2, speed, 0)                                                         'start motor at speed
    pauseMSec(interval)                                                         'setup scurve timer interval                                                         
    ActualADC := CoxaADC                                                       'get current ADC value
    if CoxaCounter == 1                                                        'check if ADC not changing
      oldADC := ActualADC                                                       'reset check ADC value
    if CoxaStopError(oldADC, ActualADC)                                        'error if ADC not changing
      dc.set(2, stopmotor, 0)
      coxaColor := Red
      if CoxaTest == 1
        io.str(0,String(13, "Coxa Done 4"))
        io.str(0,String(13, "Error: up ramp "))
        io.dec(0,ActualADC)                                    
      return                                                                    'exit ramp procedure                        
    if t2 == 8                                                                  'get initial position data after motor start
      ActualADC := CoxaADC                                                     'get current ADC value
      startADC := ActualADC                                                     'save starting ADC value
      distance := ||(ActualADC - TargetADC)                                     'absolute ADC distance to move
      case rampdirection
        1: ADChalf := ActualADC - (distance/4)                                  'get halfway mark
           'validate ramp direction is correct - ADChalf is between ActualADC and TargetADC
           if TargetADC > ActualADC
             rampdirection := 2
             dc.set(2, stopmotor, 0)
             speed := PWMcalcCoxa(t2, rampdirection)                                         'get speed for s-curve value 
             dc.set(2, speed, 0)                                                         'start motor at speed
             ADChalf := ActualADC + (distance/4)                                  'get halfway mark
             if CoxaTest == 1
               io.str(0,String(13, "Error: up rampdirection reversal "))
        2: ADChalf := ActualADC + (distance/4)                                  'get halfway mark
           if TargetADC < ActualADC
             rampdirection := 1
             dc.set(2, stopmotor, 0)
             speed := PWMcalcCoxa(t2, rampdirection)                                         'get speed for s-curve value 
             dc.set(2, speed, 0)                                                         'start motor at speed
             ADChalf := ActualADC - (distance/4)                                  'get halfway mark
             if CoxaTest == 1
               io.str(0,String(13, "Error: up rampdirection reversal "))
      if CoxaTest == 1
        io.str(0,String(13, "Start: "))
        io.str(0,String("Angle: "))
        io.dec(0,CoxaAngle)                                    
        io.str(0,String("  -ActualADC: "))
        io.dec(0,ActualADC)                                    
        io.str(0,String( "  -TargetADC: "))
        io.dec(0,TargetADC)                                    
        io.str(0,String( "  -StartADC: "))
        io.dec(0,StartADC)                                    
        io.str(0,String( "  -adcHalf: "))
        io.dec(0,adcHalf)                                    

    case rampdirection
      1: if ActualADC < ADChalf                                             'if at halfway point, stop ramp
           if CoxaTest == 1
             io.str(0,String(13, "Stop ramp: "))
             io.dec(0,CoxaAngle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -T2: "))
             io.dec(0,T2)                                    
             io.str(0,String( "  -Dir: "))
             io.dec(0,rampdirection)                                    
             io.str(0,String( "  -ADChalf: "))
            io.dec(0,ADChalf)                                    
           quit
      2: if ActualADC > ADChalf                                             'if at halfway point, stop ramp
           if CoxaTest == 1
             io.str(0,String(13, "Stop ramp: "))
             io.dec(0,CoxaAngle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -T2: "))
             io.dec(0,T2)                                    
             io.str(0,String( "  -Dir: "))
             io.dec(0,rampdirection)                                    
             io.str(0,String( "  -ADChalf: "))
             io.dec(0,ADChalf)                                    
           quit
    if CoxaTest == 1
      io.str(0,String(13, "Angle: "))
      io.dec(0,CoxaAngle)                                    
      io.str(0,String("  -ActualADC: "))
      io.dec(0,ActualADC)                                    
      io.str(0,String( "  -T2: "))
      io.dec(0,T2)                                    
      io.str(0,String( "  -Dir: "))
      io.dec(0,rampdirection)                                    
      io.str(0,String( "  -TargetADC: "))
      io.dec(0,TargetADC)                                    
   
  'run at full speed until same ADC interval (+fudge) used to start is reached
  oldADC := ActualADC
  case rampdirection
    1: adc1 := TargetADC + 250                                                  'average value for full stop
       repeat while ActualADC > adc1                                            'maintain full speed
         ActualADC := CoxaADC                                                  'get current ADC value
         pauseMSec(interval)                                                    'setup scurve timer interval
         if CoxaCounter == 1
           oldADC := ActualADC
         if CoxaStopError(oldADC, ActualADC)                                    'error if ADC not changing
           dc.set(1, stopmotor, 0)
           coxaColor := Red
           if CoxaTest == 1
             io.str(0,String(13, "Coxa Done 5"))
             io.str(0,String(13, "Error: Full speed, Dir 1 "))
             io.dec(0,ActualADC)                                    
           return
         if CoxaTest == 1  
           io.str(0,String(13, "Angle: "))
           io.dec(0,CoxaAngle)                                    
           io.str(0,String("  -ActualADC: "))
           io.dec(0,ActualADC)                                    
           io.str(0,String( "  -adc1: "))
           io.dec(0,adc1)                                    
           io.str(0,String( "  -Dir: "))
           io.dec(0,rampdirection)                                    
           io.str(0,String( "  -TargetADC: "))
           io.dec(0,TargetADC)                                    
           io.str(0,String( "  -Speed: "))
           io.dec(0,speed)                                    
    2: adc1 := TargetADC - 250                                                  'average value for full stop
       repeat while ActualADC < adc1                                            'maintain full speed
         ActualADC := CoxaADC                                                  'get current ADC value
         pauseMSec(interval)                                                    'setup scurve timer interval
         if CoxaCounter == 1
           oldADC := ActualADC
         if CoxaStopError(oldADC, ActualADC)                                    'error if ADC not changing
           dc.set(2, stopmotor, 0)
           coxaColor := Red
           if CoxaTest == 1
             io.str(0,String(13, "Coxa Done 6"))
             io.str(0,String(13, "Error: Full speed, Dir 2 "))
             io.dec(0,ActualADC)                                    
           return
         if CoxaTest == 1  
           io.str(0,String(13, "Angle: "))
           io.dec(0,CoxaAngle)                                    
           io.str(0,String("  -ActualADC: "))
           io.dec(0,ActualADC)                                    
           io.str(0,String( "  -adc1: "))
           io.dec(0,adc1)                                    
           io.str(0,String( "  -Dir: "))
           io.dec(0,rampdirection)                                    
           io.str(0,String( "  -TargetADC: "))
           io.dec(0,TargetADC)                                    
           io.str(0,String( "  -Speed: "))
           io.dec(0,speed)                                    
   
  'down ramp
'  t2 := t2 + 2                                                                  'reset counter to 0
  t2 := 0 #> t2 <# 8                                                            'constrain input 
  oldADC := ActualADC
  repeat t1 from t2 to 8                                                        'start where up ramp stopped
    speed := PWMcalcCoxa(t1, rampdirection)                                         'get speed for s-curve value 
    dc.set(2, speed, 0)
    pauseMSec(interval)                                                         'setup scurve timer interval
    ActualADC := CoxaADC                                                       'get current ADC value
    if CoxaCounter == 1
      oldADC := ActualADC
    if CoxaStopError(oldADC, ActualADC)                                    'error if ADC not changing
      dc.set(2, stopmotor, 0)
      coxaColor := Red
      if CoxaTest == 1
        io.str(0,String(13, "Coxa Done 7"))
        io.str(0,String(13, "Error: Down ramp "))
        io.dec(0,ActualADC)                                    
      return
     
    case rampdirection
      1: if ActualADC =< TargetADC                                              'at target value, stop 'motor
           speed := 1500
           dc.set(2, speed, 0)
           coxaColor := Green
           result := 0
           if CoxaTest == 1
             io.str(0,String(13, "Coxa Done 8"))
             io.str(0,String(13, "Stop: "))
             io.dec(0,CoxaAngle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -TargetADC: "))
             io.dec(0,TargetADC)                                    
             io.str(0,String( "  -Speed: "))
             io.dec(0,speed)                                    
             io.str(0,String( "  -T1: "))
             io.dec(0,t1)
           quit 
      2: if ActualADC => TargetADC                                              'at target value, stop motor
           speed := 1500
           dc.set(2, speed, 0)
           coxaColor := Green
           if CoxaTest == 1
             io.str(0,String(13, "Coxa Done 9"))
             io.str(0,String(13, "Stop: "))
             io.dec(0,CoxaAngle)                                    
             io.str(0,String("  -ActualADC: "))
             io.dec(0,ActualADC)                                    
             io.str(0,String( "  -TargetADC: "))
             io.dec(0,TargetADC)                                    
             io.str(0,String( "  -Speed: "))
             io.dec(0,speed)                                    
             io.str(0,String( "  -T1: "))
             io.dec(0,t1)
           quit 
    if CoxaTest == 1
      io.str(0,String(13, "Angle: "))
      io.dec(0,CoxaAngle)                                    
      io.str(0,String("  -ActualADC: "))
      io.dec(0,ActualADC)                                    
      io.str(0,String( "  -TargetADC: "))
      io.dec(0,TargetADC)                                    
      io.str(0,String( "  -Speed: "))
      io.dec(0,speed)                                    
      io.str(0,String( "  -T1: "))
      io.dec(0,t1)
  'ensure stop 'motor at end of rampdown
  dc.set(2, stopmotor, 0)
  coxaColor := Green
  if CoxaTest == 1
    io.str(0,String(13, "Finish: "))
    io.str(0,String("Angle: "))
    io.dec(0,CoxaAngle)                                    
    io.str(0,String("  -ActualADC: "))
    io.dec(0,ActualADC)                                    
    io.str(0,String( "  -TargetADC: "))
    io.dec(0,TargetADC)                                    
    io.str(0,String(13))
  if CoxaTest == 1
    io.str(0,String(13, "Coxa Done 10"))
   

pub CoxaStopError(iADC, aADC)
  CoxaCounter := CoxaCounter + 1                                                                    'count to 5
  if CoxaCounter == 1                                                                     'at start get ADC value
    iADC := aADC
  if CoxaCounter == 5                                                                     'max count 5
    CoxaCounter := 0
    if iADC == aADC                                                             'error, ADC not changing
      Result := true


pub PWMcalcFemur(scurv, type)
'input scurve value, output PWM value. type is 1-2 for direction and position of Scurve
'1 - 1500 to 1000 and 1000 to 1500, 2 - 1500 to 2000 and 2000 to 1500
  case type
    1:  result := scurvePWM1[scurv]
    2:  result := scurvePWM2[scurv]

pub PWMcalcTibia(scurv, type)
'input scurve value, output PWM value. type is 1-2 for direction and position of Scurve
'1 - 1500 to 1000 and 1000 to 1500, 2 - 1500 to 2000 and 2000 to 1500
  case type
    1:  result := scurvePWM1[scurv]
    2:  result := scurvePWM2[scurv]

pub PWMcalcCoxa(scurv, type)
'input scurve value, output PWM value. type is 1-2 for direction and position of Scurve
'1 - 1500 to 1000 and 1000 to 1500, 2 - 1500 to 2000 and 2000 to 1500
  case type
    1:  result := scurvePWM1[scurv]
    2:  result := scurvePWM2[scurv]

{pub GetCoxaTarget(angle, minA, maxA, minADC, maxADC)| m, b 
  'calculate ADC value using slope of line, y=mx+b where m = y1-y0/x1-x0
  m := f.fdiv(f.fsub(maxADC,minADC), f.fsub(maxA, minA))
  b := f.fsub(maxADC,f.fmul(maxA, m))
  result := f.fTrunc(f.fadd(f.fmul(m, f.ffloat(angle)), b)) 
}

pub GetADC | n, ch, ave 
  adc.start(DIO, DII, CLK, CS, 8, 3, 12, 1) 
'' DTPin: outgoing (from Controller) serial data pin (0-31); ignored if 1-channel ADC
'' INPin: incoming (to Controller) serial data pin (0-31)
'' ClkPin: serial clock pin (0-31)
'' RSPin: reset, C S pin (0-31)
'' ChCount: Number of channels on ADC (3208/3008 = 8; 3204/3004 = 4; 3202/3002 = 2; 3201/3001 = 1)
'' ActChannels: number of channels to scan (1-8)
'' BitCount: number of bits the ADC outputs (2-16)
'' SingDiff: ADC single or differential mode (1 single, 0 differential                                                       ' start adc driver
  repeat
    repeat ch from 0 to 2
      longfill(@array, 0, 10)                                                     'clear array
      ave := 0                                                     'read 3 channels
      repeat n from 0 to 9                                                      'read 9 values into array
        array[n] := adc.getval(ch)
      Sort(@array, 10)                                                      'sort array
      repeat n from 3 to 6                                                      'drop low and high from array
        ave += array[n]                                                          'total remaining values
      case ch
        0: femurADC := ave >>= 2                                                'divide by 4 
        1: tibiaADC := ave >>= 2
        2: coxaADC  := ave >>= 2


pub Sort (arrayAddr, arrayLength) | j1, i1, k1, val
  'sort input array ascending
  arrayLength--
  repeat i1 from 1 to arrayLength
    val := long[arrayAddr][i1]
    j1 := i1
    k1~
    repeat while (long[arrayAddr][--j1] > val) 
      if ++k1 == i1
        quit
    if k1
      longmove(j1 := arrayAddr + (i1-k1+1) <<2, j1-1, k1)
    long[arrayAddr][i1-k1] := val

pub Str2Dec(str) | index
  ' from FullDuplexSerial
  result := 0
  repeat index from 0 to (strsize(str) -1)
    result *= 10
    result += byte[str][index] - "0"
    
PUB pauseMSec(Duration)
'' Pause execution in milliseconds.
'' Duration = number of milliseconds to delay
  waitcnt(((clkfreq / 1_000 * Duration - 3932) #> 381) + cnt)


dat
femurMinAngle word 37
femurMaxAngle word 138
femurMinADC word 185
femurMaxADC word 3640

tibiaMinAngle word 90
tibiaMaxAngle word 150
tibiaMinADC word 140
tibiaMaxADC word 4010

coxaMinAngle word 59
coxaMaxAngle word 120
coxaMinADC word 1600
coxaMaxADC word 2250

'SCurve value is PWM value
scurvePWM1 word 1000,1014,1052,1108,1176,1250,1324,1392,1448 
scurvePWM2 word 2000,1986,1948,1892,1824,1750,1676,1608,1552


{{ Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                  